package arvores;

import ine5408.arvores.AVL;
import ine5408.cst.Lado;
import ine5408.excecoes.ExcecaoEstruturaVazia;
import ine5408.excecoes.ExcecaoOperacaoIlegal;
import ine5408.pp.Visitante;

public class ArvoreAVL<C extends Comparable<C>, T> implements AVL<C, T> {

	private C chaveRaiz;
	private T dadoRaiz;
	private int tamanho;
	private AVL<C, T> subDireita;
	private AVL<C, T> subEsquerda;

	public boolean ehFolha() {
		boolean ehFolha = false;

		if (!estaVazia())
			ehFolha = subEsquerda.estaVazia() && subDireita.estaVazia();
		
		return ehFolha;
	}

	public boolean estaCheia() {
		return false;
	}

	public boolean estaVazia() {
		return dadoRaiz == null;
	}

	public void esvazie() {
		dadoRaiz = null;
		chaveRaiz = null;
		subEsquerda = null;
		subDireita = null;
		tamanho = 0;

	}

	public void insira(C chave, T dado) throws ExcecaoOperacaoIlegal {
		if (estaVazia()) {
			chaveRaiz = chave;
			dadoRaiz = dado;
			subDireita = new ArvoreAVL<C, T>();
			subEsquerda = new ArvoreAVL<C, T>();
		} else {

			if (chave == chaveRaiz)
				throw new ExcecaoOperacaoIlegal();
			if (chave.compareTo(chaveRaiz) == -1)
				subEsquerda.insira(chave, dado);
			else
				subDireita.insira(chave, dado);
		}

		// TODO Onde está a verificação de balanceamento?
		tamanho++;
	}

	public void recebaVisitanteEmLargura(Visitante<T> arg0) {
		// TODO Auto-generated method stub

	}

	public void recebaVisitanteEmOrdem(Visitante<T> v) {
		// TODO Auto-generated method stub

	}

	public void recebaVisitanteEmPosOrdem(Visitante<T> v) {
		// TODO Auto-generated method stub

	}

	public void recebaVisitanteEmPreOrdem(Visitante<T> v) {
		// TODO Auto-generated method stub

	}

	public void recebaVisitanteEmProfundidade(Visitante<T> arg0) {
		// TODO Auto-generated method stub

	}

	public T remova(C chave) throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		return null;
	}

	public T retorne(C chave) {
		T dado = null;

		if (chaveRaiz.equals(chave))
			dado = dadoRaiz;

		return dado;
	}

	public int retorneAltura() {
		int altura = -1;

		if (!estaVazia())
			altura = 1 + Math.max(subEsquerda.retorneAltura(), subDireita
					.retorneAltura());
		return altura;
	}

	public int retorneAlturaSubarvore(Lado l) throws ExcecaoOperacaoIlegal {
		if (estaVazia())
			throw new ExcecaoOperacaoIlegal();

		int alturaSub = subEsquerda.retorneAltura();
		if (l == Lado.DIR)
			alturaSub = subDireita.retorneAltura();
		return alturaSub;
	}

	public int retorneGrau() {
		return 2;
	}

	public T retorneRaiz() throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		return dadoRaiz;
	}

	public int retorneTamanho() {
		return tamanho;
	}

}
