package arvores;

import ine5408.cst.Lado;
import ine5408.excecoes.ExcecaoEstruturaVazia;
import ine5408.pp.Visitante;

public class ArvoreBinaria<T> implements ine5408.arvores.ArvoreBinaria<T> {

	private T raiz;
	private ine5408.arvores.ArvoreBinaria<T> arvoreEsq;
	private ine5408.arvores.ArvoreBinaria<T> arvoreDir;
	private ine5408.arvores.ArvoreBinaria<T> pai;

	public ArvoreBinaria(ine5408.arvores.ArvoreBinaria<T> pai) {
		this.pai = pai;
	}

	public boolean ehFolha() {
		return !estaVazia() && arvoreEsq.estaVazia() && arvoreDir.estaVazia();
	}

	public boolean estaCheia() {
		return false;
	}

	public boolean estaVazia() {
		return raiz == null;
	}

	public void esvazie() {
		raiz = null;
		arvoreDir = null;
		arvoreEsq = null;
	}

	public void recebaVisitanteEmLargura(Visitante<T> visitante) {
		// TODO Auto-generated method stub
	}

	public void recebaVisitanteEmProfundidade(Visitante<T> visitante) {
		recebaVisitanteEmProfundidadePreOrdem(visitante);
	}

	public void recebaVisitanteEmProfundidadeEmOrdem(Visitante<T> visitante) {
		if (!estaVazia() && !visitante.satisfeito()) {
			arvoreEsq.recebaVisitanteEmProfundidadeEmOrdem(visitante);
			visitante.visite(raiz);
			arvoreDir.recebaVisitanteEmProfundidadeEmOrdem(visitante);
		}

	}

	public void recebaVisitanteEmProfundidadePosOrdem(Visitante<T> visitante) {
		if (!estaVazia() && !visitante.satisfeito()) {
			arvoreEsq.recebaVisitanteEmProfundidadePosOrdem(visitante);
			arvoreDir.recebaVisitanteEmProfundidadePosOrdem(visitante);
			visitante.visite(raiz);
		}

	}

	public void recebaVisitanteEmProfundidadePreOrdem(Visitante<T> visitante) {
		if (!estaVazia() && !visitante.satisfeito()) {
			visitante.visite(raiz);
			arvoreEsq.recebaVisitanteEmProfundidadePreOrdem(visitante);
			arvoreDir.recebaVisitanteEmProfundidadePreOrdem(visitante);
		}

	}

	public ine5408.arvores.ArvoreBinaria<T> removaSubarvore(Lado lado)
			throws ExcecaoEstruturaVazia {
		if (this.estaVazia())
			throw new ExcecaoEstruturaVazia();

		ine5408.arvores.ArvoreBinaria<T> retorno = retorneSubarvore(lado);
		ine5408.arvores.ArvoreBinaria<T> vazia = new ArvoreBinaria<T>(this);
		substituaSubarvore(vazia, lado);

		return retorno;
	}

	public int retorneAltura() {
		int altura = -1;
		if (!estaVazia())
			altura = 1 + Math.max(arvoreDir.retorneAltura(), arvoreEsq
					.retorneAltura());
		return altura;
	}

	public int retorneGrau() {
		return 2;
	}

	public ine5408.arvores.ArvoreBinaria<T> retornePai() {
		return pai;
	}

	public T retorneRaiz() throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		return raiz;
	}

	public ine5408.arvores.ArvoreBinaria<T> retorneSubarvore(Lado l)
			throws ExcecaoEstruturaVazia {

		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		ine5408.arvores.ArvoreBinaria<T> retornada = arvoreEsq;
		if (l == Lado.DIR)
			retornada = arvoreDir;

		return retornada;
	}

	public int retorneTamanho() {
		int tamanho = 0;

		if (!estaVazia())
			tamanho = 1 + arvoreEsq.retorneTamanho()
					+ arvoreDir.retorneTamanho();

		return tamanho;
	}

	public void substituaRaiz(T dado) {

		if (estaVazia()) {
			arvoreEsq = new ArvoreBinaria<T>(this);
			arvoreDir = new ArvoreBinaria<T>(this);
		}

		raiz = dado;
	}

	public void substituaSubarvore(ine5408.arvores.ArvoreBinaria<T> subArvore,
			Lado lado) throws ExcecaoEstruturaVazia {
		if (this.estaVazia())
			throw new ExcecaoEstruturaVazia();

		if (lado == Lado.DIR)
			arvoreDir = subArvore;
		else
			arvoreEsq = subArvore;
	}

}
