package arvores;

import ine5408.excecoes.ExcecaoEstruturaVazia;
import ine5408.excecoes.ExcecaoPosicaoIlegal;
import ine5408.pp.Iterador;
import ine5408.pp.Visitante;
import lista.ListaDuplamenteEncadeada;

public class ArvoreGeral<T> implements ine5408.arvores.ArvoreGeral<T> {

	private T raiz;
	private int grau;
	private ListaDuplamenteEncadeada<ine5408.arvores.ArvoreGeral<T>> subArvores = new ListaDuplamenteEncadeada<ine5408.arvores.ArvoreGeral<T>>();
	private ine5408.arvores.ArvoreGeral<T> pai;

	public ArvoreGeral(ine5408.arvores.ArvoreGeral<T> pai) {
		this.pai = pai;
		grau = -1;
	}

	public boolean ehFolha() {
		return !estaVazia() && subArvores.estaVazia();
	}

	public boolean estaCheia() {
		return false;
	}

	public boolean estaVazia() {
		return raiz == null;
	}

	public void esvazie() {
		raiz = null;
		subArvores = null;
		grau = -1;

	}

	public void insiraArvoreNaPosicao(ine5408.arvores.ArvoreGeral<T> subArvore,
			int posicao) throws ExcecaoEstruturaVazia, ExcecaoPosicaoIlegal {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		if (posicao < 1 || posicao > grau + 1)
			throw new ExcecaoPosicaoIlegal(posicao);
		subArvores.insiraNaPosicao(subArvore, posicao);
		grau++;

	}

	public void insiraArvoreNoFim(ine5408.arvores.ArvoreGeral<T> subArvore)
			throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		subArvores.insiraNoFim(subArvore);
		grau++;
	}

	public void insiraArvoreNoInicio(ine5408.arvores.ArvoreGeral<T> subArvore)
			throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		subArvores.insiraNoInicio(subArvore);
		grau++;

	}

	public void recebaVisitanteEmLargura(Visitante<T> v) {
		ListaDuplamenteEncadeada<ine5408.arvores.ArvoreGeral<T>> lista = new ListaDuplamenteEncadeada<ine5408.arvores.ArvoreGeral<T>>();

		v.visite(raiz);
		
		for (int i = 0; i < subArvores.retorneTamanho(); i++) {
			lista.insiraNoFim(subArvores.retorneDaPosicao(i+1));
		}

		for (Iterador<ine5408.arvores.ArvoreGeral<T>> iter = lista
				.retorneIterador(); iter.temMais() && !v.satisfeito();) {
			ine5408.arvores.ArvoreGeral<T> arvoreSendoVisitada = iter
					.retorneProximo();
			v.visite(arvoreSendoVisitada.retorneRaiz());
			for (int i = 0; i < arvoreSendoVisitada.retorneGrau(); i++) {
				lista
						.insiraNoFim(arvoreSendoVisitada
								.retorneArvoreDaPosicao(i+1));
			}
		}
	}

	public void recebaVisitanteEmProfundidade(Visitante<T> v) {
		if (!estaVazia() && !v.satisfeito()) {
			v.visite(raiz);
			
			for (Iterador<ine5408.arvores.ArvoreGeral<T>> iter = subArvores
					.retorneIterador(); iter.temMais();)
				iter.retorneProximo().recebaVisitanteEmProfundidade(v);
		}
	}

	public ine5408.arvores.ArvoreGeral<T> removaArvoreDaPosicao(int posicao)
			throws ExcecaoEstruturaVazia, ExcecaoPosicaoIlegal {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		if (posicao < 1 || posicao > grau)
			throw new ExcecaoPosicaoIlegal(posicao);
		grau--;
		return subArvores.removaDaPosicao(posicao);
	}

	public ine5408.arvores.ArvoreGeral<T> removaArvoreDoFim()
			throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		grau--;
		return subArvores.removaDoFim();
	}

	public ine5408.arvores.ArvoreGeral<T> removaArvoreDoInicio()
			throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		grau--;
		return subArvores.removaDoInicio();
	}

	public int retorneAltura() {
		int altura = -1;

		if (!estaVazia()) {
			int max = -1, atual;
			for (int i = 1; i <= subArvores.retorneTamanho(); i++) {
				atual = subArvores.retorneDaPosicao(i).retorneAltura();
				if (atual > max)
					max = atual;
			}
			altura = 1 + max;
		}

		return altura;
	}

	public ine5408.arvores.ArvoreGeral<T> retorneArvoreDaPosicao(int posicao)
			throws ExcecaoEstruturaVazia, ExcecaoPosicaoIlegal {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		if (posicao < 1 || posicao > grau)
			throw new ExcecaoPosicaoIlegal(posicao);
		return subArvores.retorneDaPosicao(posicao);
	}

	public ine5408.arvores.ArvoreGeral<T> retorneArvoreDoFim()
			throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		return subArvores.retorneDoFim();
	}

	public ine5408.arvores.ArvoreGeral<T> retorneArvoreDoInicio()
			throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		return subArvores.retorneDoInicio();
	}

	public int retorneGrau() {
		return grau;
	}

	public ine5408.arvores.ArvoreGeral<T> retornePai() {
		return pai;
	}

	public T retorneRaiz() throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		return raiz;
	}

	public int retorneTamanho() {
		int tamanho = 0;
		int tamanhoDasSubarvores = 0;
		if (!estaVazia()) {
			for (int i = 0; i < subArvores.retorneTamanho(); i++)
				tamanhoDasSubarvores += subArvores.retorneDaPosicao(i + 1)
						.retorneTamanho();
			tamanho = 1 + tamanhoDasSubarvores;
		}

		return tamanho;
	}

	public void substituaArvoreDaPosicao(
			ine5408.arvores.ArvoreGeral<T> subArvore, int posicao)
			throws ExcecaoEstruturaVazia, ExcecaoPosicaoIlegal {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		if (posicao < 1 || posicao > grau)
			throw new ExcecaoPosicaoIlegal(posicao);
		subArvores.substituaDaPosicao(subArvore, posicao);

	}

	public void substituaArvoreDoFim(ine5408.arvores.ArvoreGeral<T> subArvore)
			throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		subArvores.substituaDoFim(subArvore);

	}

	public void substituaArvoreDoInicio(ine5408.arvores.ArvoreGeral<T> subArvore)
			throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();
		subArvores.substituaDoInicio(subArvore);

	}

	public void substituaRaiz(T dado) {
		if (grau == -1)
			grau = 0;
		raiz = dado;
	}

}
