package lista;

import ine5408.excecoes.ExcecaoEstruturaCheia;
import ine5408.excecoes.ExcecaoEstruturaVazia;
import ine5408.excecoes.ExcecaoOperacaoIlegal;
import ine5408.excecoes.ExcecaoPosicaoIlegal;
import ine5408.lineares.Lista;
import ine5408.pp.Visitante;

public class ListaEncadeadaRecursiva<T> implements Lista<T> {

	@SuppressWarnings("hiding")
	private class IteradorDeLER<T> implements ine5408.pp.Iterador<T> {

		private ListaEncadeadaRecursiva<T> listaAtual;

		private IteradorDeLER(ListaEncadeadaRecursiva<T> lista) {
			listaAtual = lista;
		}

		public T retorneProximo() throws ExcecaoOperacaoIlegal {
			if (listaAtual.estaVazia())
				throw new ExcecaoOperacaoIlegal("Não há mais itens na lista");
			final T dado = listaAtual.retorneDoInicio();
			listaAtual = listaAtual.resto;
			return dado;
		}

		public boolean temMais() {
			return !listaAtual.estaVazia();
		}
	}

	T dado;
	ListaEncadeadaRecursiva<T> resto;

	public ListaEncadeadaRecursiva() {
		dado = null;
		resto = null;
	}

	private ListaEncadeadaRecursiva(T dado, ListaEncadeadaRecursiva<T> resto) {
		this.dado = dado;
		this.resto = resto;
	}

	@SuppressWarnings("unchecked")
	public boolean equals(Object outro) {
		boolean retorno = false;
		if (outro instanceof ListaEncadeadaRecursiva) {
			final ListaEncadeadaRecursiva<T> outraLista = (ListaEncadeadaRecursiva<T>) outro;

			if (estaVazia() && outraLista.estaVazia())
				retorno = true;
			else
				retorno = this.dado.equals(outraLista.dado)
						&& this.resto.equals(outraLista.resto);
		}
		return retorno;
	}

	public boolean estaCheia() {
		return false;
	}

	public boolean estaVazia() {
		return dado == null;
	}

	public void esvazie() {
		this.dado = null;
		resto = null;

	}

	public void insiraNaPosicao(T dado, int pos) throws ExcecaoEstruturaCheia,
			ExcecaoPosicaoIlegal {
		if (pos < 1)
			throw new ExcecaoPosicaoIlegal(pos);

		if (pos == 1)
			this.insiraNoInicio(dado);
		else {
			if (estaVazia())
				throw new ExcecaoPosicaoIlegal(pos);
			resto.insiraNaPosicao(dado, pos - 1);
		}

	}

	public void insiraNoFim(T dado) throws ExcecaoEstruturaCheia {
		if (this.estaVazia()) {
			this.dado = dado;
			resto = new ListaEncadeadaRecursiva<T>();
		} else
			resto.insiraNoFim(dado);
	}

	public void insiraNoInicio(T dado) throws ExcecaoEstruturaCheia {
		resto = new ListaEncadeadaRecursiva<T>(this.dado, this.resto);
		this.dado = dado;
	}

	public boolean pertence(T dado) {
		boolean retorno = false;
		if (!estaVazia())
			if (this.dado == dado)
				retorno = true;
			else
				retorno = resto.pertence(dado);

		return retorno;
	}

	public void recebaVisitante(Visitante<T> visitante) {
		if (!this.estaVazia()) {
			visitante.visite(dado);
			if (!visitante.satisfeito())
				resto.recebaVisitante(visitante);
		}

	}

	public T removaDaPosicao(int pos) throws ExcecaoEstruturaVazia,
			ExcecaoPosicaoIlegal {
		T dadoRemovido;
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		if (pos == 1)
			dadoRemovido = this.removaDoInicio();
		else if (resto.estaVazia())
			throw new ExcecaoPosicaoIlegal(pos);
		else
			dadoRemovido = resto.removaDaPosicao(pos - 1);

		return dadoRemovido;
	}

	public T removaDoFim() throws ExcecaoEstruturaVazia {
		T dadoRemovido;
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		if (resto.estaVazia()) {
			dadoRemovido = this.dado;
			this.dado = null;
		} else
			dadoRemovido = resto.removaDoFim();

		return dadoRemovido;
	}

	public T removaDoInicio() throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		final T removido = dado;

		if (!resto.estaVazia()) {
			dado = resto.dado;
			resto = resto.resto;
		} else
			esvazie();

		return removido;
	}

	public Lista<T> removaIguais(T dado) throws ExcecaoEstruturaVazia {
		Lista<T> listaDeIguais = new ListaEncadeadaRecursiva<T>();

		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		if (this.dado.equals(dado)) {
			listaDeIguais.insiraNoInicio(this.removaDoInicio());

			if (!this.estaVazia()) {
				final Lista<T> iguaisDoResto = this.removaIguais(dado);

				iguaisDoResto.insiraNoInicio(listaDeIguais.retorneDoInicio());
				listaDeIguais = iguaisDoResto;
			}

		} else if (!resto.estaVazia())
			listaDeIguais = resto.removaIguais(dado);

		return listaDeIguais;
	}

	public T removaIgual(T dado) throws ExcecaoEstruturaVazia {
		T retorno = null;
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		if (this.dado.equals(dado)) {
			retorno = dado;
			this.removaDoInicio();
		} else if (!resto.estaVazia())
			retorno = resto.removaIgual(dado);

		return retorno;
	}

	public T removaPenultimo() throws ExcecaoOperacaoIlegal {
		T removido = null;
		if (this.estaVazia() || resto.estaVazia())
			throw new ExcecaoOperacaoIlegal();
		if (resto.resto.estaVazia()) {
			removido = dado;
			dado = resto.dado;
			resto = null;
		} else
			removido = resto.removaPenultimo();
		return removido;
	}

	public T retorneDaPosicao(int pos) throws ExcecaoEstruturaVazia,
			ExcecaoPosicaoIlegal {
		T dadoRetornado;
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		if (pos == 1)
			dadoRetornado = dado;
		else if (resto.estaVazia())
			throw new ExcecaoPosicaoIlegal(pos);
		else
			dadoRetornado = resto.retorneDaPosicao(pos - 1);

		return dadoRetornado;

	}

	public T retorneDoFim() throws ExcecaoEstruturaVazia {
		T dadoRetornado;
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		if (resto.estaVazia())
			dadoRetornado = dado;
		else
			dadoRetornado = resto.retorneDoFim();

		return dadoRetornado;
	}

	public T retorneDoInicio() throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		return this.dado;
	}

	public IteradorDeLER<T> retorneIterador() {
		return new IteradorDeLER<T>(this);
	}

	public T retornePenultimo() throws ExcecaoOperacaoIlegal {

		T dadoRetornado;
		if (this.estaVazia() || resto.estaVazia())
			throw new ExcecaoOperacaoIlegal();

		if (resto.resto.estaVazia())
			dadoRetornado = dado;
		else
			dadoRetornado = resto.retornePenultimo();

		return dadoRetornado;
	}

	public int retorneTamanho() {
		int tamanho = 0;

		if (dado != null)
			tamanho = 1 + resto.retorneTamanho();

		return tamanho;
	}

	public void substituaDaPosicao(T dado, int pos)
			throws ExcecaoEstruturaVazia, ExcecaoPosicaoIlegal {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		if (pos == 1)
			this.dado = dado;
		else if (resto.estaVazia())
			throw new ExcecaoPosicaoIlegal(pos);
		else
			resto.substituaDaPosicao(dado, pos - 1);
	}

	public void substituaDoFim(T dado) throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		if (resto.estaVazia())
			this.dado = dado;
		else
			resto.substituaDoFim(dado);
	}

	public void substituaDoInicio(T dado) throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		this.dado = dado;
	}

	public Lista<T> temIguais(T dado) {
		Lista<T> listaDeIguais = new ListaEncadeadaRecursiva<T>();

		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		if (this.dado.equals(dado))
			listaDeIguais.insiraNoInicio(this.dado);

		if (!resto.estaVazia()) {
			final Lista<T> iguaisDoResto = resto.temIguais(dado);
			if (!listaDeIguais.estaVazia())
				iguaisDoResto.insiraNoInicio(this.dado);
			listaDeIguais = iguaisDoResto;
		}
		return listaDeIguais;
	}

	public T temIgual(T dado) {
		T resposta = null;

		if (!estaVazia())
			if (this.dado.equals(dado))
				resposta = dado;
			else
				resposta = resto.temIgual(dado);

		return resposta;
	}
}
