package tabelaDeHash;

import ine5408.excecoes.ExcecaoEstruturaVazia;
import ine5408.excecoes.ExcecaoOperacaoIlegal;
import ine5408.lineares.Lista;
import ine5408.pp.Iterador;
import ine5408.pp.Visitante;
import ine5408.th.FuncaoG;
import ine5408.util.Associacao;

import java.lang.reflect.Array;

import lista.ListaEncadeadaRecursiva;

public class TabelaDeHash<C extends Comparable<C>, T> implements
		ine5408.th.TabelaDeHash<C, T> {

	Lista<Associacao<C, T>>[] array;
	private int numeroDeDados;
	private FuncaoG fG;

	@SuppressWarnings("unchecked")
	public TabelaDeHash(FuncaoG funcao) {
		fG = funcao;
		array = (Lista<Associacao<C, T>>[]) Array.newInstance(
				new ListaEncadeadaRecursiva<Associacao<C, T>>().getClass(), fG
						.getM());
	}

	public boolean estaCheia() {
		return false;
	}

	public boolean estaVazia() {
		return numeroDeDados == 0;
	}

	@SuppressWarnings("unchecked")
	public void esvazie() {
		array = (Lista<Associacao<C, T>>[]) Array.newInstance(
				new ListaEncadeadaRecursiva<Associacao<C, T>>().getClass(), fG
						.getM());
		numeroDeDados = 0;
	}

	public int f(C chave) {
		return chave.hashCode();
	}

	public int g(int inteiro) {
		return fG.getValor(inteiro);
	}

	public int h(C chave) {
		return g(f(chave));
	}

	public void insira(C chave, T dado) throws ExcecaoOperacaoIlegal {
		Lista<Associacao<C, T>> lista = array[h(chave)];
		Associacao<C, T> associacao = new Associacao<C, T>(chave, dado);

		if (lista == null)
			lista = array[h(chave)] = new ListaEncadeadaRecursiva<Associacao<C, T>>();
		else if (lista.temIgual(associacao) != null)
			throw new ExcecaoOperacaoIlegal();

		lista.insiraNoInicio(associacao);
		numeroDeDados++;
	}

	public void recebaVisitante(Visitante<T> visitante) {
		Lista<Associacao<C, T>> listaAtual;

		for (int i = 0; i < array.length && !visitante.satisfeito(); i++) {
			listaAtual = array[i];

			if (listaAtual != null && !visitante.satisfeito())
				for (Iterador<Associacao<C, T>> iter = listaAtual
						.retorneIterador(); iter.temMais()
						&& !visitante.satisfeito();) {
					Associacao<C, T> par = iter.retorneProximo();
					visitante.visite(par.getDado());
				}
		}

	}

	public T remova(C chave) throws ExcecaoEstruturaVazia {
		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		T retorno = null;
		Lista<Associacao<C, T>> lista = array[h(chave)];

		if (lista != null) {
			retorno = array[h(chave)].removaIgual(
					new Associacao<C, T>(chave, null)).getDado();
			numeroDeDados--;
		}

		return retorno;
	}

	public T retorne(C chave) {
		T retorno = null;
		Lista<Associacao<C, T>> lista = array[h(chave)];

		if (lista != null)
			for (Iterador<Associacao<C, T>> iterator = lista.retorneIterador(); iterator
					.temMais()
					&& retorno == null;) {
				Associacao<C, T> atual = iterator.retorneProximo();
				if (atual.getChave().equals(chave))
					retorno = atual.getDado();
			}

		return retorno;
	}

	public Lista<C> retorneChaves() {
		Lista<C> listaDeChaves = new ListaEncadeadaRecursiva<C>();

		for (Lista<Associacao<C, T>> lista : array)
			if (lista != null)
				for (Iterador<Associacao<C, T>> iter = lista.retorneIterador(); iter
						.temMais();) {
					Associacao<C, T> par = iter.retorneProximo();
					listaDeChaves.insiraNoInicio(par.getChave());
				}

		return listaDeChaves;
	}

	public Lista<T> retorneDados() {
		Lista<T> listaDeDados = new ListaEncadeadaRecursiva<T>();

		for (Lista<Associacao<C, T>> lista : array)
			if (lista != null)
				for (Iterador<Associacao<C, T>> iter = lista.retorneIterador(); iter
						.temMais();) {
					Associacao<C, T> par = iter.retorneProximo();
					listaDeDados.insiraNoInicio(par.getDado());
				}

		return listaDeDados;
	}

	public int retorneM() {
		return fG.getM();
	}

	public int retorneTamanho() {
		return numeroDeDados;
	}

	public int[] retorneTamanhoColisoesPorEntrada() {
		int posicoesNoArray = fG.getM();
		int[] retorno = new int[posicoesNoArray];

		for (int i = 0; i < posicoesNoArray; i++)
			if (array[i] != null)
				retorno[i] = array[i].retorneTamanho();

		return retorno;
	}

	public void substitua(C chave, T dado) throws ExcecaoEstruturaVazia,
			ExcecaoOperacaoIlegal {

		if (estaVazia())
			throw new ExcecaoEstruturaVazia();

		boolean trocou = false;
		Lista<Associacao<C, T>> lista = array[h(chave)];

		if (lista != null)
			for (Iterador<Associacao<C, T>> iter = lista.retorneIterador(); iter
					.temMais()
					&& !trocou;) {
				Associacao<C, T> atual = iter.retorneProximo();
				if (atual.getChave().equals(chave)) {
					atual.setDado(dado);
					trocou = true;
				}
			}

		if (!trocou)
			throw new ExcecaoOperacaoIlegal();
	}
}