package testes.arvoreAVL;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import ine5408.arvores.AVL;
import ine5408.cst.Lado;
import ine5408.excecoes.ExcecaoEstruturaVazia;
import ine5408.excecoes.ExcecaoOperacaoIlegal;

import org.junit.Before;
import org.junit.Test;

import arvores.ArvoreAVL;

public class TestesDeAVL {

	private AVL<Integer, String> avl;

	@Test
	public void aALturaDeUmaArvoreComUmDadoEhZero() {
		avl.insira(5, "boi");
		assertEquals(0, avl.retorneAltura());
	}

	@Test
	public void aALturaDeUmaArvoreVaziaEhMenosUm() {
		assertEquals(-1, avl.retorneAltura());
	}

	@Test
	public void aAlturaDeUmaSubarvoreDeArvoreComUmDadoEhMenosUm() {
		avl.insira(5, "boi");

		assertEquals(-1, avl.retorneAlturaSubarvore(Lado.ESQ));
		assertEquals(-1, avl.retorneAlturaSubarvore(Lado.DIR));
	}

	@Test
	public void alturaDeArvoreEsvaziadaEhMenosUm() {
		avl.insira(5, "boi");
		avl.esvazie();

		assertEquals(-1, avl.retorneAltura());
	}

	@Test
	public void aoInserirUmDadoEsteSeTornaARaiz() {
		avl.insira(5, "boi");
		assertEquals("boi", avl.retorneRaiz());
	}

	@Test
	public void aposSerEsvaziadaAVLEstaVazia() {
		avl.insira(5, "boi");
		avl.insira(4, "boi");

		avl.esvazie();

		assertTrue(avl.estaVazia());
	}

	@Test
	public void aposSerEsvaziadaAVLTemTamanhoZero() {
		avl.insira(5, "boi");
		avl.insira(4, "boi");

		avl.esvazie();

		assertEquals(0, avl.retorneTamanho());
	}

	@Test
	public void arvoreComUmDadoEhFolha() {
		avl.insira(5, "boi");

		assertTrue(avl.ehFolha());
	}

	@Test
	public void AVLComDoisDadosTemAlturaDois() {
		avl.insira(5, "boi");
		avl.insira(4, "boi");
		assertEquals(2, avl.retorneTamanho());
	}

	@Test
	public void AVLComNDadosTemTamanhoN() {
		int n = 100;

		for (int i = 0; i < n; i++)
			avl.insira(i, "boi");

		assertEquals(n, avl.retorneTamanho());

	}

	@Test
	public void AVLComUmDadoTemAlturaUm() {
		avl.insira(5, "boi");
		assertEquals(1, avl.retorneTamanho());
	}

	@Test
	public void AVLInicialmenteEstaVazia() {
		assertTrue(avl.estaVazia());
	}

	@Test
	public void AVLNuncaEstaCheia() {
		assertFalse(avl.estaCheia());
	}

	@Test
	public void AVLVaziaNaoEhFolha() {
		assertFalse(avl.ehFolha());
	}

	@Test
	public void GrauDaAVLEhSempreDois() {
		assertEquals(2, avl.retorneGrau());
	}

	@Before
	public void inicia() {
		avl = new ArvoreAVL<Integer, String>();
	}

	@Test
	public void inserirDadoComChaveMaiorQueARaizAumentaAALturaDaArvoreDireita() {
		avl.insira(1, "menor");
		avl.insira(10, "maior");

		assertEquals(0, avl.retorneAlturaSubarvore(Lado.DIR));
		assertEquals(-1, avl.retorneAlturaSubarvore(Lado.ESQ));

	}

	@Test
	public void inserirDadoComChaveMenorQueARaizAumentaAALturaDaArvoreEsquerda() {
		avl.insira(10, "maior");
		avl.insira(1, "menor");

		assertEquals(0, avl.retorneAlturaSubarvore(Lado.ESQ));
		assertEquals(-1, avl.retorneAlturaSubarvore(Lado.DIR));
	}

	@Test(expected = ExcecaoOperacaoIlegal.class)
	public void inserirDoisDadosComMesmaChaveGeraExcecao() {
		avl.insira(5, "boi");
		avl.insira(5, "boi");
	}

	@Test(expected = ExcecaoOperacaoIlegal.class)
	public void obterAlturaDeSubarvoreDirGeraExcecaoSeARaizEstaVazia() {
		avl.retorneAlturaSubarvore(Lado.DIR);
	}

	@Test(expected = ExcecaoOperacaoIlegal.class)
	public void obterAlturaDeSubarvoreEsqGeraExcecaoSeARaizEstaVazia() {
		avl.retorneAlturaSubarvore(Lado.ESQ);
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void RemoverDadoDeAVLVaziaGeraExcecao() {
		avl.remova(10);
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void RetornarRaizDeAVLVaziaGeraExcecao() {
		avl.retorneRaiz();
	}

	@Test
	public void retorneDadoRetornaODadoAssociadoAChaveDada() {
		avl.insira(5, "boi");
		assertEquals("boi", avl.retorne(5));
	}

	@Test
	public void seInserirDoisDadosARaizEhSempreOPrimeiro() {
		avl.insira(5, "boi");
		avl.insira(4, "vaca");

		assertEquals("boi", avl.retorneRaiz());
	}

	@Test
	public void TamanhoInicialDaAVLEhZero() {
		assertEquals(0, avl.retorneTamanho());
	}
}
