package testes.arvoreBinaria;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import ine5408.cst.Lado;
import ine5408.excecoes.ExcecaoEstruturaVazia;

import java.util.Random;

import org.junit.Before;
import org.junit.Test;

import testes.VisitanteConcatenador;
import arvores.ArvoreBinaria;

public class TestesArvoreBinaria {

	private ine5408.arvores.ArvoreBinaria<String> a;

	@Test
	public void aAlturaInicialDeUmaArvoreEhMenosUm() {
		assertEquals(-1, a.retorneAltura());
	}

	@Test
	public void aArvoreBinariaNuncaEstaCheia() {
		assertFalse(a.estaCheia());
	}

	@Test
	public void alturaDeUmaArvoreComUmDadoEhZero() {
		a.substituaRaiz("3");
		assertEquals(0, a.retorneAltura());
	}

	@Test
	public void aoSubstituirARaizDeUmaArvoreVaziaEstaNaoEstaMaisVazia() {
		assertTrue(a.estaVazia());
		a.substituaRaiz("10");
		assertFalse(a.estaVazia());
	}

	@Test
	public void aoSubstituirARaizDeUmaArvoreVaziaEstaSeTornaARaiz() {
		a.substituaRaiz("10");
		assertEquals("10", a.retorneRaiz());
	}

	@Test
	public void arvoreBinariaVaziaNaoEhFolha() {
		assertFalse(a.ehFolha());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void arvoreVaziaLancaExcecaoAoTentarObterARaiz() {
		a.retorneRaiz();

	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void arvoreVaziaLancaExcecaoAoTentarObterSubArvoreDireita() {
		a.retorneSubarvore(Lado.DIR);

	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void arvoreVaziaLancaExcecaoAoTentarObterSubArvoreEsquerda() {
		a.retorneSubarvore(Lado.ESQ);

	}

	@Before
	public void inicia() {
		a = new ArvoreBinaria<String>(null);
	}

	@Test
	public void inicialmenteArvoreBinariaEstaVazia() {
		assertTrue(a.estaVazia());
	}

	@Test
	public void oGrauDaArvoreBinariaEhDois() {
		assertEquals(2, a.retorneGrau());
	}

	private void preencherArvoreCom7DadosBalanceados() {
		ine5408.arvores.ArvoreBinaria<String> esq, dir;

		a.substituaRaiz("1");
		(esq = a.retorneSubarvore(Lado.ESQ)).substituaRaiz("2");
		(dir = a.retorneSubarvore(Lado.DIR)).substituaRaiz("3");
		esq.retorneSubarvore(Lado.ESQ).substituaRaiz("4");
		esq.retorneSubarvore(Lado.DIR).substituaRaiz("5");
		dir.retorneSubarvore(Lado.ESQ).substituaRaiz("6");
		dir.retorneSubarvore(Lado.DIR).substituaRaiz("7");
	}

	@Test
	public void removaSubArvoreInsereArvoreVaziaNoLugar() {
		a.substituaRaiz("1");
		ine5408.arvores.ArvoreBinaria<String> subDir = a
				.retorneSubarvore(Lado.DIR);
		subDir.substituaRaiz("5");
		a.removaSubarvore(Lado.DIR);
		assertTrue(a.retorneSubarvore(Lado.DIR).estaVazia());
	}

	@Test
	public void removaSubArvoreRetornaArvoreRemovida() {
		a.substituaRaiz("1");
		ine5408.arvores.ArvoreBinaria<String> subDir = a
				.retorneSubarvore(Lado.DIR);
		subDir.substituaRaiz("5");
		assertEquals(subDir, a.removaSubarvore(Lado.DIR));
	}

	@Test
	public void retorneSubArvoreDeArvoreNaoVaziaFunciona() {
		a.substituaRaiz("3");
		ine5408.arvores.ArvoreBinaria<String> b = new ArvoreBinaria<String>(a);
		a.substituaSubarvore(b, Lado.DIR);
		assertEquals(b, a.retorneSubarvore(Lado.DIR));
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void retorneSubArvoreDeArvoreVaziaLancaExcecao() {
		a.retorneSubarvore(Lado.DIR);
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void substituirSubArvoreDeSubArvoreVaziaLancaExcecao() {
		a.substituaRaiz("2");
		ine5408.arvores.ArvoreBinaria<String> b = new ArvoreBinaria<String>(a);
		a.substituaSubarvore(b, Lado.ESQ);
		ine5408.arvores.ArvoreBinaria<String> c = new ArvoreBinaria<String>(b);
		b.substituaSubarvore(c, Lado.ESQ);
	}

	@Test
	public void substituirSubArvoreDeUmaArvoreNaoVaziaTemEfeito() {
		a.substituaRaiz("2");
		ine5408.arvores.ArvoreBinaria<String> b = new ArvoreBinaria<String>(a);
		a.substituaSubarvore(b, Lado.ESQ);
		ine5408.arvores.ArvoreBinaria<String> c = new ArvoreBinaria<String>(a);
		a.substituaSubarvore(c, Lado.DIR);
		assertEquals(b, a.retorneSubarvore(Lado.ESQ));
		assertEquals(c, a.retorneSubarvore(Lado.DIR));
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void substituirSubarvoreDeUmaArvoreVaziaLancaExcecao() {
		a.substituaSubarvore(new ArvoreBinaria<String>(null), Lado.ESQ);
	}

	@Test
	public void testeRecebaVisitanteEmProfundidade() {
		testeRecebaVisitanteEmProfundidadePreOrdem();
	}

	@Test
	public void testeRecebaVisitanteEmProfundidadeEmOrdem() {
		preencherArvoreCom7DadosBalanceados();

		VisitanteConcatenador visitante = new VisitanteConcatenador();
		assertEquals("", visitante.retorneDados());

		a.recebaVisitanteEmProfundidadeEmOrdem(visitante);
		assertEquals("4251637", visitante.retorneDados());
	}

	@Test
	public void testeRecebaVisitanteEmProfundidadePosOrdem() {
		preencherArvoreCom7DadosBalanceados();

		VisitanteConcatenador visitante = new VisitanteConcatenador();
		assertEquals("", visitante.retorneDados());

		a.recebaVisitanteEmProfundidadePosOrdem(visitante);
		assertEquals("4526731", visitante.retorneDados());
	}

	@Test
	public void testeRecebaVisitanteEmProfundidadePreOrdem() {
		preencherArvoreCom7DadosBalanceados();

		VisitanteConcatenador visitante = new VisitanteConcatenador();
		assertEquals("", visitante.retorneDados());
		a.recebaVisitanteEmProfundidadePreOrdem(visitante);
		assertEquals("1245367", visitante.retorneDados());
	}

	@Test
	public void umaArvoreComDadosTodosNaDireitaTemAlturaIgualANumeroDeDadosMenos1() {
		int numDeDados = (new Random()).nextInt(50);

		final ine5408.arvores.ArvoreBinaria<String> b = a;

		for (int i = 1; i <= numDeDados; i++) {
			a.substituaRaiz("3");
			a = a.retorneSubarvore(Lado.DIR);
		}

		assertEquals(numDeDados - 1, b.retorneAltura());
	}

	@Test
	public void umaArvoreComDadosTodosNaEsquerdaTemAlturaIgualANumeroDeDadosMenos1() {
		int numDeDados = (new Random()).nextInt(50);

		final ine5408.arvores.ArvoreBinaria<String> b = a;

		for (int i = 1; i <= numDeDados; i++) {
			a.substituaRaiz("3");
			a = a.retorneSubarvore(Lado.ESQ);
		}

		assertEquals(numDeDados - 1, b.retorneAltura());
	}

	@Test
	public void umaArvoreComDoisDadosNaoEhFolha() {
		a.substituaRaiz("3");
		a.substituaSubarvore(a, Lado.ESQ);
		assertFalse(a.ehFolha());
	}

	@Test
	public void umaArvoreComSubArvoresEsvaziadaEstaVazia() {
		a.substituaRaiz("5");
		a.retorneSubarvore(Lado.DIR).substituaRaiz("5");

		a.esvazie();
		assertTrue(a.estaVazia());

	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void umaArvoreComSubArvoresEsvaziadaLancaExcecaoAoRetornarSubARvore() {
		a.substituaRaiz("5");
		a.retorneSubarvore(Lado.DIR).substituaRaiz("5");

		a.esvazie();
		a.retorneSubarvore(Lado.DIR);

	}

	@Test
	public void umaArvoreComUmDadoEDepoisEsvaziadaEstaVazia() {
		a.substituaRaiz("5");
		assertFalse(a.estaVazia());

		a.esvazie();
		assertTrue(a.estaVazia());

	}

	@Test
	public void umaArvoreComUmDadoEhFolha() {
		a.substituaRaiz("5");
		assertTrue(a.ehFolha());

	}

	@Test
	public void umaArvoreComUmDadoESubArvoreComUmDadoTemTamanhoDois() {
		a.substituaRaiz("1");
		a.retorneSubarvore(Lado.DIR).substituaRaiz("5");

		assertEquals(2, a.retorneTamanho());
	}

	@Test
	public void umaArvoreComUmDadoESubArvoresComDadosTemTamanhoTres() {
		a.substituaRaiz("1");
		a.retorneSubarvore(Lado.DIR).substituaRaiz("5");
		a.retorneSubarvore(Lado.ESQ).substituaRaiz("5");

		assertEquals(3, a.retorneTamanho());
	}

	@Test
	public void umaArvoreComUmDadoESubSubArvoreComDadoTemTamanhoTres() {
		a.substituaRaiz("1");
		ine5408.arvores.ArvoreBinaria<String> subDir = a
				.retorneSubarvore(Lado.DIR);
		subDir.substituaRaiz("5");
		subDir.retorneSubarvore(Lado.DIR).substituaRaiz("5");

		assertEquals(3, a.retorneTamanho());
	}

	@Test
	public void umaArvoreComUmDadoTemDuasSubArvoresVazia() {
		a.substituaRaiz("10");

		assertTrue(a.retorneSubarvore(Lado.ESQ).estaVazia());
		assertTrue(a.retorneSubarvore(Lado.DIR).estaVazia());
	}

	@Test
	public void umaArvoreComUmDadoTemTamanhoUm() {
		a.substituaRaiz("0");
		assertEquals(1, a.retorneTamanho());
	}

	@Test
	public void umaArvoreEhCriadaVazia() {
		assertTrue(a.estaVazia());
	}

	@Test
	public void umaArvoreNaoVaziaTemPai() {
		a.substituaRaiz("2");
		a.retorneSubarvore(Lado.DIR).substituaRaiz("4");
		assertEquals(a, a.retorneSubarvore(Lado.DIR).retornePai());
	}

	@Test
	public void umaArvoreNuncaDiminuiDeAlturaAoSeInsrerirUmDado() {
		final Random r = new Random();

		a.substituaRaiz("3");
		final ine5408.arvores.ArvoreBinaria<String> b = a;
		int alturaOriginal = a.retorneAltura();

		for (int i = 0; i < 100; i++) {
			int n = r.nextInt(2);
			Lado l = Lado.values()[n];

			a.substituaRaiz("3");
			a = a.retorneSubarvore(l);
		}

		assertTrue(b.retorneAltura() >= alturaOriginal);
	}

	@Test
	public void umaArvoreNuncaEstaCheia() {
		ine5408.arvores.ArvoreBinaria<String> b = a;

		for (int i = 0; i < 10000; i++) {
			a.substituaRaiz(String.valueOf(i));
			a = a.retorneSubarvore(Lado.DIR);
		}

		assertFalse(b.estaCheia());
	}

	@Test
	public void umaArvoreVaziaNaoTemPai() {
		assertEquals(null, a.retornePai());
	}

	@Test
	public void umaArvoreVaziaTemTamanhoZero() {
		assertEquals(0, a.retorneTamanho());
	}

}
