package testes.arvoreGeral;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import ine5408.excecoes.ExcecaoEstruturaVazia;

import org.junit.Before;
import org.junit.Test;

import testes.VisitanteConcatenador;
import arvores.ArvoreGeral;

public class TestesArvoreGeral {

	private ine5408.arvores.ArvoreGeral<String> a;

	private ine5408.arvores.ArvoreGeral<String> inserirNvacas(int n) {
		ine5408.arvores.ArvoreGeral<String> b = a;
		for (int i = 0; i < n; i++) {
			ine5408.arvores.ArvoreGeral<String> arvore = new ArvoreGeral<String>(
					a);
			arvore.substituaRaiz("vaca");

			a.insiraArvoreNoInicio(arvore);
			a = a.retorneArvoreDoInicio();
		}
		return b;
	}

	@Before
	public void prepararTestes() {
		a = new ArvoreGeral<String>(null);
	}

	@Test
	public void testeEhFolha() {
		a.substituaRaiz("boi");
		assertTrue(a.ehFolha());
	}

	@Test
	public void testeEstaCheia() {
		assertFalse(a.estaCheia());
		a.substituaRaiz("boi");

		ine5408.arvores.ArvoreGeral<String> b = inserirNvacas(1000);

		assertFalse(b.estaCheia());
	}

	@Test
	public void testeEstaVazia() {
		assertTrue(a.estaVazia());

		a.substituaRaiz("borat");

		assertFalse(a.estaVazia());
	}

	@Test
	public void testeEsvazie() {
		assertTrue(a.estaVazia());

		a.substituaRaiz("boi");
		ine5408.arvores.ArvoreGeral<String> b = inserirNvacas(500);

		assertFalse(b.estaVazia());
		b.esvazie();
		assertTrue(b.estaVazia());
	}

	@Test
	public void testeQueArvoreCom2ElementosNaoEhFolha() {
		a.substituaRaiz("boi");
		ine5408.arvores.ArvoreGeral<String> b = inserirNvacas(1);

		assertFalse(b.ehFolha());
	}

	@Test
	public void testeQueArvoreVaziaNaoEhFolha() {
		assertFalse(a.ehFolha());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeQueRetorneRaizDeArvoreVaziaGeraExcecao() {
		a.retorneRaiz();
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeQueSubstituaArvoreDoFimComArvoreVaziaGeraExcecao() {
		a.substituaArvoreDoFim(new ArvoreGeral<String>(null));
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeQueSubstituaArvoreDoInicioComArvoreVaziaGeraExcecao() {
		a.substituaArvoreDoInicio(new ArvoreGeral<String>(null));
	}

	@Test
	public void testeRetorneAltura() {
		assertEquals(-1, a.retorneAltura());
		a.substituaRaiz("pamela");
		assertEquals(0, a.retorneAltura());

		ine5408.arvores.ArvoreGeral<String> esq, dir;
		(esq = new ArvoreGeral<String>(a)).substituaRaiz("borat");
		(dir = new ArvoreGeral<String>(a)).substituaRaiz("oxanna");
		a.insiraArvoreNoInicio(esq);
		a.insiraArvoreNoFim(dir);

		assertEquals(1, a.retorneAltura());
	}

	@Test
	public void testeRetorneGrau() {
		assertEquals(-1, a.retorneGrau());

		a.substituaRaiz("boi");
		assertEquals(0, a.retorneGrau());

		ArvoreGeral<String> nova = new ArvoreGeral<String>(a);
		nova.substituaRaiz("vaca");
		a.insiraArvoreNoInicio(nova);

		assertEquals(1, a.retorneGrau());

	}

	@Test
	public void testeRetorneRaiz() {
		a.substituaRaiz("pamela");
		assertEquals("pamela", a.retorneRaiz());
	}

	@Test
	public void testeRetorneTamanho() {
		assertEquals(0, a.retorneTamanho());
		a.substituaRaiz("boi");

		ine5408.arvores.ArvoreGeral<String> b = inserirNvacas(50);

		assertEquals(51, b.retorneTamanho());
	}

	@Test
	public void testeSubstituaArvoreDoFim() {
		a.substituaRaiz("boi");
		ine5408.arvores.ArvoreGeral<String> nova = new ArvoreGeral<String>(a);
		nova.substituaRaiz("vaca");
		a.insiraArvoreNoFim(nova);

		assertEquals("vaca", a.retorneArvoreDoFim().retorneRaiz());

		a.substituaArvoreDoFim(new ArvoreGeral<String>(a));
		assertTrue(a.retorneArvoreDoFim().estaVazia());
	}

	@Test
	public void testeSubstituaArvoreDoInicio() {
		a.substituaRaiz("boi");
		ine5408.arvores.ArvoreGeral<String> nova = new ArvoreGeral<String>(a);
		nova.substituaRaiz("vaca");
		a.insiraArvoreNoInicio(nova);

		assertEquals("vaca", a.retorneArvoreDoInicio().retorneRaiz());

		a.substituaArvoreDoInicio(new ArvoreGeral<String>(a));
		assertTrue(a.retorneArvoreDoInicio().estaVazia());
	}

	@Test
	public void testeSubstituaRaiz() {
		assertTrue(a.estaVazia());
		a.substituaRaiz("boi");
		assertEquals("boi", a.retorneRaiz());

		a.substituaRaiz("vaca");
		assertEquals("vaca", a.retorneRaiz());
	}

	@Test
	public void testInsiraArvoreNaPosicao() {
		a.substituaRaiz("a");
		ine5408.arvores.ArvoreGeral<String> arvore = new ArvoreGeral<String>(a);
		arvore.substituaRaiz("b");
		a.insiraArvoreNaPosicao(arvore, 1);
		assertEquals(arvore, a.retorneArvoreDaPosicao(1));
	}

	// @Test
	public void testInsiraArvoreNoFim() {
		// TODO Teste vazio
	}

	// @Test
	public void testInsiraArvoreNoInicio() {
		// TODO Teste vazio
	}

	@Test
	public void testRecebaVisitanteEmLargura() {
		a.substituaRaiz("a");		
		ine5408.arvores.ArvoreGeral<String> arvore = new ArvoreGeral<String>(a);
		ine5408.arvores.ArvoreGeral<String> arvore2 = new ArvoreGeral<String>(a);
		ine5408.arvores.ArvoreGeral<String> arvore3 = new ArvoreGeral<String>(a);
		arvore.substituaRaiz("b");
		arvore2.substituaRaiz("c");
		arvore3.substituaRaiz("d");
		a.insiraArvoreNaPosicao(arvore, 1);
		a.insiraArvoreNaPosicao(arvore2, 2);
		a.insiraArvoreNaPosicao(arvore3, 3);
		ine5408.arvores.ArvoreGeral<String> arvore4 = new ArvoreGeral<String>(arvore);
		ine5408.arvores.ArvoreGeral<String> arvore5 = new ArvoreGeral<String>(arvore2);
		arvore4.substituaRaiz("e");
		arvore5.substituaRaiz("f");
		arvore.insiraArvoreNaPosicao(arvore4, 1);
		arvore2.insiraArvoreNaPosicao(arvore5, 1);
		
		VisitanteConcatenador visitante = new VisitanteConcatenador();
		assertEquals("", visitante.retorneDados());

		a.recebaVisitanteEmLargura(visitante);
		assertEquals("abcdef", visitante.retorneDados());
	}

	@Test
	public void testeRecebaVisitanteEmProfundidade() {
		ine5408.arvores.ArvoreGeral<String> um, dois, tres, quatro, cinco, seis, sete;
		(um = new ArvoreGeral<String>(null)).substituaRaiz("1");
		(dois = new ArvoreGeral<String>(um)).substituaRaiz("2");
		(tres = new ArvoreGeral<String>(um)).substituaRaiz("3");
		(quatro = new ArvoreGeral<String>(dois)).substituaRaiz("4");
		(cinco = new ArvoreGeral<String>(dois)).substituaRaiz("5");
		(seis = new ArvoreGeral<String>(tres)).substituaRaiz("6");
		(sete = new ArvoreGeral<String>(tres)).substituaRaiz("7");

		tres.insiraArvoreNoFim(seis);
		tres.insiraArvoreNoFim(sete);
		dois.insiraArvoreNoFim(quatro);
		dois.insiraArvoreNoFim(cinco);
		um.insiraArvoreNoFim(dois);
		um.insiraArvoreNoFim(tres);

		VisitanteConcatenador vc = new VisitanteConcatenador();
		um.recebaVisitanteEmProfundidade(vc);

		assertEquals("1245367", vc.retorneDados());
	}

	// @Test
	public void testRemovaArvoreDaPosicao() {
		// TODO Teste vazio
	}

	// @Test
	public void testRemovaArvoreDoFim() {
		// TODO Teste vazio
	}

	// @Test
	public void testRemovaArvoreDoInicio() {
		// TODO Teste vazio
	}

	// @Test
	public void testRetorneArvoreDaPosicao() {
		// TODO Teste vazio
	}

	// @Test
	public void testRetorneArvoreDoFim() {
		// TODO Teste vazio
	}

	// @Test
	public void testRetorneArvoreDoInicio() {
		// TODO Teste vazio
	}

	// @Test
	public void testRetornePai() {
		// TODO Teste vazio
	}

	// @Test
	public void testSubstituaArvoreDaPosicao() {
		// TODO Teste vazio
	}

}
