package testes.filaBaseadaEmLER;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import fila.FilaBaseadaEmLER;
import ine5408.excecoes.ExcecaoEstruturaVazia;
import ine5408.lineares.Fila;

import org.junit.Before;
import org.junit.Test;

import testes.VisitanteConcatenador;

public class TestesProprios {

	private Fila<String> fila;

	@Before
	public void prepararTestes() {
		fila = new FilaBaseadaEmLER<String>();
	}

	@Test
	public void testeEstaCheia() {
		for (int i = 0; i < 1000; i++) {
			fila.insira("oi");
			fila.insira("tudo");
			fila.insira("bem");
		}
		assertFalse(fila.estaCheia());
	}

	@Test
	public void testeEstaVazia() {
		assertTrue(fila.estaVazia());
		fila.insira("oi");
		assertFalse(fila.estaVazia());
		fila.remova();
		assertTrue(fila.estaVazia());
	}

	@Test
	public void testeEsvazie() {
		fila.insira("oi");
		fila.insira("tudo");
		fila.insira("bem");
		assertFalse(fila.estaVazia());
		fila.esvazie();
		assertTrue(fila.estaVazia());
	}

	@Test
	public void testeInsira() {
		fila.insira("oi");
		fila.insira("tudo");
		fila.remova();
		assertEquals("tudo", fila.retorneDoInicio());
		assertEquals("tudo", fila.retorneDoFim());
	}

	@Test
	public void testeRecebaVisitante() {
		VisitanteConcatenador v = new VisitanteConcatenador();
		fila.insira("inicio");
		fila.insira("meio");
		fila.insira("fim");
		fila.recebaVisitante(v);
		assertEquals("iniciomeiofim", v.retorneDados());
	}

	@Test
	public void testeRemova() {
		fila.insira("oi");
		fila.insira("tudo");
		fila.insira("bem");
		assertEquals("oi", fila.remova());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRemovaDeFilaVaziaLancaExcecao() {
		fila.remova();
	}

	@Test
	public void testeRetorneDoFim() {
		fila.insira("oi");
		fila.insira("tudo");
		fila.insira("bem");
		assertEquals("bem", fila.retorneDoFim());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRetorneDoFimDeFilaVaziaLancaExcecao() {
		fila.retorneDoFim();
	}

	@Test
	public void testeRetorneDoInicio() {
		fila.insira("oi");
		fila.insira("tudo");
		fila.insira("bem");
		assertEquals("oi", fila.retorneDoInicio());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRetorneDoInicioDeFilaVaziaLancaExcecao() {
		fila.retorneDoInicio();
	}

	@Test
	public void testeRetorneTamanho() {
		for (int i = 0; i < 10; i++)
			fila.insira(String.valueOf(i));
		assertEquals(10, fila.retorneTamanho());
	}
}