package testes.filaCircular;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import ine5408.excecoes.ExcecaoEstruturaCheia;
import ine5408.excecoes.ExcecaoEstruturaVazia;

import org.junit.Before;
import org.junit.Test;

import testes.VisitanteConcatenador;
import fila.FilaCircular;

public class TestesDeFilaCircular {

	private static final int tamanho = 10;
	private FilaCircular<String> filaTestada;

	@Before
	public void prepararTestes() {
		filaTestada = new FilaCircular<String>(tamanho);
	}

	@Test
	public void testeEstaCheia() {
		for (int i = 0; i < tamanho; i++)
			filaTestada.insira(String.valueOf(i));

		assertTrue(filaTestada.estaCheia());
	}

	@Test
	public void testeEstaVazia() {
		assertTrue(filaTestada.estaVazia());
	}

	@Test
	public void testeEsvazie() {
		filaTestada.insira("oxanna");
		filaTestada.insira("bilo");
		filaTestada.insira("pamela");
		assertFalse(filaTestada.estaVazia());

		filaTestada.esvazie();

		assertTrue(filaTestada.estaVazia());
	}

	@Test
	public void testeInsira() {
		filaTestada.insira("a");
		filaTestada.insira("b");
		filaTestada.insira("c");
		assertEquals("c", filaTestada.retorneDoFim());
	}

	@Test(expected = ExcecaoEstruturaCheia.class)
	public void testeQueInserirEmFilaCheiaLancaExcecao() {
		for (int i = 0; i < tamanho; i++)
			filaTestada.insira(String.valueOf(i));

		filaTestada.insira("dado extra");
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeQueRemovaDeUmaFilaVaziaLancaExcecao() {
		filaTestada.remova();
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeQueRetorneDoFimDeUmaFilaVaziaLancaExcecao() {
		filaTestada.remova();
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeQueRetorneDoInicioDeUmaFilaVaziaLancaExcecao() {
		filaTestada.remova();
	}

	@Test
	public void testeRecebaVisitante() {
		VisitanteConcatenador v = new VisitanteConcatenador();

		filaTestada.insira("inicio");
		filaTestada.insira("meio");
		filaTestada.insira("fim");
		filaTestada.recebaVisitante(v);
		assertEquals("iniciomeiofim", v.retorneDados());
	}

	@Test
	public void testeRemova() {
		filaTestada.insira("borat");
		filaTestada.insira("natalia");
		filaTestada.insira("oxanna");

		assertEquals("borat", filaTestada.retorneDoInicio());

		filaTestada.remova();

		assertEquals("natalia", filaTestada.retorneDoInicio());
	}

	@Test
	public void testeRetorneDoFim() {
		for (int i = 0; i < tamanho; i++)
			filaTestada.insira(String.valueOf(i));
		filaTestada.remova();
		assertEquals("1", filaTestada.retorneDoInicio());

		filaTestada.insira("natalia");

		assertEquals("natalia", filaTestada.retorneDoFim());
	}

	@Test
	public void testeRetorneDoInicio() {
		for (int i = 0; i < tamanho; i++)
			filaTestada.insira(String.valueOf(i));
		filaTestada.remova();
		filaTestada.remova();

		assertEquals("2", filaTestada.retorneDoInicio());
	}

	@Test
	public void testeRetorneTamanho() {
		filaTestada.insira("a");
		filaTestada.insira("b");
		filaTestada.insira("c");
		assertEquals(3, filaTestada.retorneTamanho());
	}

}
