package testes.listaDuplamenteEncadeada;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import ine5408.excecoes.ExcecaoEstruturaVazia;
import ine5408.excecoes.ExcecaoPosicaoIlegal;
import ine5408.lineares.Lista;
import ine5408.pp.Iterador;
import lista.ListaDuplamenteEncadeada;

import org.junit.Before;
import org.junit.Test;

public class TestesProprios {

	private Lista<String> lista;
	private Lista<String> lista2;

	@Test
	public void aListaCriadaEstaVazia() {
		assertTrue(lista.estaVazia());
	}

	@Test
	public void aListaNaoEstaVaziaSeContemDados() {
		lista.insiraNoInicio("oi");
		assertFalse(lista.estaVazia());
	}

	@Test
	public void duasListasSaoIguaisSeTemOsMesmosElementosNasMesmasPosicoes() {
		lista.insiraNoFim("vaca");
		lista.insiraNoFim("boi");

		lista2.insiraNoFim("vaca");

		assertFalse(lista.equals(lista2));

		lista2.insiraNoFim("boi");

		assertEquals(lista, lista2);

		lista2.insiraNoInicio("touro");
		assertFalse(lista.equals(lista2));

	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void inserirEmPosicaoMaiorQueTamanhoLancaExcecao() {
		lista.insiraNoInicio("um");
		lista.insiraNoFim("dois");

		lista.insiraNaPosicao("dez", 10);
	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void inserirEmPosicaoMenorQueUmLancaExcecao() {
		lista.insiraNoInicio("abc");
		lista.insiraNaPosicao("zero", 0);
	}

	@Test
	public void inserirNaPosicaoUmDeUmaListaVaziaÉPossível() {
		lista.insiraNaPosicao("boi", 1);
		assertEquals("boi", lista.retorneDoInicio());
	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void naoSePodeRetornarDePosicaoInexistente() {
		lista.insiraNoInicio("asda");
		lista.retorneDaPosicao(50);
	}

	@Before
	public void prepararTestes() {
		lista = new ListaDuplamenteEncadeada<String>();
		lista2 = new ListaDuplamenteEncadeada<String>();
	}

	@Test
	public void removerIguaisRetornaUmaListaDosIguaisRemovidos() {
		lista.insiraNoInicio("abc");
		lista.insiraNoInicio("ddd");
		lista.insiraNoInicio("abc");

		lista2.insiraNoInicio("abc");
		lista2.insiraNoInicio("abc");

		assertEquals(lista2, lista.removaIguais("abc"));
	}

	@Test
	public void testeEsvazie() {
		lista.insiraNoFim("123");
		lista.insiraNoFim("cba");
		assertFalse(lista.estaVazia());

		lista.esvazie();
		assertTrue(lista.estaVazia());
	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void testeExcecaoQuandoInserirEmPosicaoMaiorQueTamanho() {
		lista.insiraNoInicio("vaca");
		lista.insiraNoInicio("boi");
		lista.insiraNaPosicao("pasto", 10);
	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void testeExcecaoQuandoInserirEmPosicaoMenorQueUm() {
		lista.insiraNaPosicao("cat", 0);
	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void testeExcecaoQuandoInserirEmPosicaoNegativa() {
		lista.insiraNaPosicao("cat", -1);
	}

	@Test
	public void testeInsiraNaPosicao() {
		lista.insiraNoInicio("world");
		lista.insiraNoInicio("hello");

		lista.insiraNaPosicao("good", 2);
		assertEquals("good", lista.retorneDaPosicao(2));
	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void testeInsiraNaPosicaoEmPosicaoIlegalThrowsExcecao() {
		lista.insiraNoInicio("boi");
		lista.insiraNaPosicao("vaca", 3);
	}

	@Test
	public void testeInsiraNoFim() {
		lista.insiraNoFim("hello");
		lista.insiraNoFim("world");
		assertEquals("world", lista.retorneDoFim());

		lista.insiraNoFim("again");
		assertEquals("again", lista.retorneDoFim());
	}

	@Test
	public void testeInsiraNoInicio() {
		lista.insiraNoInicio("world");
		lista.insiraNoInicio("hello");
		assertEquals("hello", lista.retorneDoInicio());

		lista.insiraNoInicio("YA");
		assertEquals("YA", lista.retorneDoInicio());
	}

	@Test
	public void testeQueAListaCriadaEstaVazia() {
		assertTrue(lista.estaVazia());
		lista.insiraNoInicio("oi");
		assertFalse(lista.estaVazia());
		lista.removaDoInicio();
		assertTrue(lista.estaVazia());
	}

	@Test
	public void testeQueAListaNuncaEstaCheia() {
		assertFalse(lista.estaCheia());

		for (int i = 0; i < 10000; i++)
			lista.insiraNoInicio("vaca");
		assertFalse(lista.estaCheia());

	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void testeQueInsiraNaPosicaoMenorQueUmLancaExcecao() {
		lista.insiraNaPosicao("bilo", 0);
	}

	@Test
	public void testeQueNãoTemIguais() {
		lista.insiraNoInicio("gato");
		lista2.esvazie();

		assertEquals(lista2, lista.temIguais("cachorro"));
	}

	@Test
	public void testeQuePertence() {
		assertFalse(lista.pertence("xyz"));
		lista.insiraNoFim("abc");
		lista.insiraNoFim("def");
		lista.insiraNoFim("xyz");
		assertTrue(lista.pertence("xyz"));
	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void testeQueRemovaDaPosicaoMenorQueUmLancaExcecao() {
		lista.insiraNoInicio("foo");
		lista.removaDaPosicao(0);
	}

	@Test
	public void testeRemovaDaPosicao() {
		lista.insiraNoFim("a");
		lista.insiraNoFim("b");
		lista.insiraNoFim("c");

		lista.removaDaPosicao(2);
		assertEquals("a", lista.retorneDaPosicao(1));
		assertEquals("c", lista.retorneDaPosicao(2));
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRemovaDaPosicaoDeListaVaziaThrowsExcecao() {
		lista.removaDaPosicao(3);
	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void testeRemovaDaPosicaoEmPosicaoIlegalThrowsExcecao() {
		lista.insiraNoInicio("vaca");
		lista.removaDaPosicao(3);
	}

	@Test
	public void testeRemovaDoFim() {
		lista.insiraNoFim("abc");
		lista.insiraNoFim("ddd");

		lista.removaDoFim();

		assertEquals("abc", lista.retorneDoFim());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRemovaDoFimDeListaVaziaThrowsExcecao() {
		lista.removaDoFim();
	}

	@Test
	public void testeRemovaDoInicio() {
		lista.insiraNoInicio("ddd");
		lista.insiraNoFim("abc");

		lista.removaDoInicio();

		assertEquals("abc", lista.retorneDoInicio());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRemovaDoInicioDeListaVaziaThrowsExcecao() {
		lista.removaDoInicio();
	}

	@Test
	public void testeRemovaIguais() {
		lista.insiraNoInicio("abc");
		lista.insiraNoInicio("ddd");
		lista.insiraNoInicio("abc");

		lista.removaIguais("abc");

		assertEquals("ddd", lista.retorneDoInicio());
		assertEquals(null, lista.temIgual("abc"));
		assertTrue(lista.retorneTamanho() == 1);
	}

	@Test
	public void testeRemovaIgual() {
		lista.insiraNoInicio("qwerty");
		lista.insiraNoInicio("asdfgh");

		assertEquals("qwerty", lista.removaIgual("qwerty"));
		assertEquals(null, lista.removaIgual("qwerty"));
		assertEquals(null, lista.removaIgual("not in the list"));

		lista2.insiraNoInicio("a");
		lista2.removaIgual("a");
		assertTrue(lista2.retorneTamanho() == 0);
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRemoverIguaisDeListaVaziaThrowsExcecao() {
		lista.removaIguais("abc");
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRemoverIgualDeListaVaziaThrowsExcecao() {
		lista.removaIgual("abc");
	}

	@Test
	public void testeRetorneDaPosicao() {
		lista.insiraNoFim("a");
		lista.insiraNoFim("b");
		lista.insiraNoFim("c");

		assertEquals("b", lista.retorneDaPosicao(2));
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRetorneDaPosicaoDeListaVaziaThrowsExcecao() {
		lista.retorneDaPosicao(2);
	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void testeRetorneDaPosicaoEmPosicaoIlegalThrowsExcecao() {
		lista.insiraNoInicio("dado");
		lista.retorneDaPosicao(3);
	}

	@Test
	public void testeRetorneDoFim() {
		lista.insiraNoFim("123");
		lista.insiraNoFim("cba");

		assertEquals(lista.retorneDoFim(), "cba");
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRetorneDoFimDeListaVaziaThrowsExcecao() {
		lista.retorneDoFim();
	}

	@Test
	public void testeRetorneDoInicio() {
		lista.insiraNoInicio("def");
		lista.insiraNoInicio("abc");

		assertEquals(lista.retorneDoInicio(), "abc");
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRetorneDoInicioDeListaVaziaThrowsExcecao() {
		lista.retorneDoInicio();
	}

	@Test
	public void testeRetorneIterador() {
		lista.insiraNoInicio("borat");
		lista.insiraNoInicio("natalia");

		Iterador<String> iter = lista.retorneIterador();

		assertTrue(iter.temMais());

		assertEquals("natalia", iter.retorneProximo());
		assertEquals("borat", iter.retorneProximo());

		assertFalse(iter.temMais());

		lista.esvazie();
		lista.insiraNoInicio("pamela");

		iter = lista.retorneIterador();
		assertEquals("pamela", iter.retorneProximo());
		assertFalse(iter.temMais());
	}

	@Test
	public void testeRetorneTamanho() {
		lista.insiraNoInicio("1");
		lista.insiraNoInicio("2");
		lista.insiraNoInicio("3");

		assertTrue(lista.retorneTamanho() == 3);
	}

	@Test
	public void testeSubstituaDaPosicao() {
		lista.insiraNoFim("a");
		lista.insiraNoFim("b");
		lista.insiraNoFim("c");

		assertEquals("b", lista.retorneDaPosicao(2));

		lista.substituaDaPosicao("novo", 2);

		assertEquals("novo", lista.retorneDaPosicao(2));
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeSubstituaDaPosicaoDeListaVaziaThrowsExcecao() {
		lista.substituaDaPosicao("boi", 3);
	}

	@Test(expected = ExcecaoPosicaoIlegal.class)
	public void testeSubstituaDaPosicaoDePosicaoIlegalThrowsExcecao() {
		lista.insiraNoInicio("vaca");
		lista.substituaDaPosicao("boi", 3);
	}

	@Test
	public void testeSubstituaDoFim() {
		lista.insiraNoFim("a");
		lista.insiraNoFim("b");

		assertEquals("b", lista.retorneDoFim());

		lista.substituaDoFim("novoFim");

		assertEquals("novoFim", lista.retorneDoFim());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeSubstituaDoFimDeListaVaziaThrowsExcecao() {
		lista.substituaDoFim("boi");
	}

	@Test
	public void testeSubstituaDoInicio() {

		lista.insiraNoInicio("a");
		lista.insiraNoFim("b");
		lista.insiraNoFim("c");

		assertEquals("a", lista.retorneDoInicio());

		lista.substituaDoInicio("novo");

		assertEquals("novo", lista.retorneDoInicio());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeSubstituaDoInicioDeListaVaziaThrowsExcecao() {
		lista.substituaDoInicio("boi");
	}

	@Test
	public void testeTemIguais() {
		lista.insiraNoInicio("boi");
		lista.insiraNoInicio("vaca");
		lista.insiraNoInicio("vaca");

		lista2.insiraNoInicio("vaca");
		lista2.insiraNoInicio("vaca");

		assertEquals(lista2, lista.temIguais("vaca"));
	}

	@Test
	public void testeTemIgual() {
		lista.insiraNoFim("abc");
		assertEquals("abc", lista.temIgual("abc"));

		assertEquals(null, lista.temIgual("non ecsiste"));
	}

}
