package testes.listaEncadeadaRecursiva;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import ine5408.excecoes.ExcecaoEstruturaVazia;
import ine5408.excecoes.ExcecaoOperacaoIlegal;
import ine5408.excecoes.ExcecaoPosicaoIlegal;
import ine5408.lineares.Lista;
import ine5408.pp.Iterador;
import lista.ListaEncadeadaRecursiva;

import org.junit.Before;
import org.junit.Test;

import testes.VisitanteConcatenador;

public class TestesDanielF {

	private Lista<String> lista;
	private VisitanteConcatenador visitante;

	@Before
	public void prepararTestes() {
		lista = new ListaEncadeadaRecursiva<String>();
		visitante = new VisitanteConcatenador();
	}

	@Test
	public void testeEstaCheia() {
		assertFalse(lista.estaCheia());
		lista.insiraNoInicio("A");
		assertFalse(lista.estaCheia());
	}

	@Test
	public void testeEstaVazia() {
		assertTrue(lista.estaVazia());
		lista.insiraNoInicio("A");
		assertFalse(lista.estaVazia());
		lista.removaDoInicio();
		assertTrue(lista.estaVazia());
	}

	@Test
	public void testeEsvazie() {
		lista.insiraNoInicio("A");
		lista.esvazie();
		assertTrue(lista.estaVazia());
	}

	@Test
	public void testeInsiraNaPosicao() {
		lista.insiraNoFim("A");
		lista.insiraNoFim("C");
		lista.insiraNaPosicao("B", 2);
		lista.recebaVisitante(visitante);
		assertEquals("ABC", visitante.retorneDados());
	}

	@Test
	public void testeInsiraNaPosicaoExcecao() {
		try {
			lista.insiraNaPosicao("A", 2);
			lista.insiraNaPosicao("B", -1);
			fail("Posicao ilegal");
		} catch (final ExcecaoPosicaoIlegal e) {
		}
	}

	@Test
	public void testeInsiraNoFim() {
		lista.insiraNoFim("A");
		lista.insiraNoFim("B");
		lista.insiraNoFim("C");
		lista.recebaVisitante(visitante);
		assertEquals("ABC", visitante.retorneDados());
	}

	@Test
	public void testeInsiraNoInicio() {
		lista.insiraNoInicio("A");
		lista.insiraNoInicio("B");
		lista.insiraNoInicio("C");
		lista.recebaVisitante(visitante);
		assertEquals("CBA", visitante.retorneDados());
	}

	@Test
	public void testePertence() {
		lista.insiraNoInicio("A");
		lista.insiraNoInicio("B");
		assertFalse(lista.pertence("C"));
		lista.insiraNaPosicao("C", 2);
		assertTrue(lista.pertence("C"));
	}

	@Test
	public void testeRecebaVisitante() {
		lista.insiraNoInicio("A");
		lista.insiraNoFim("C");
		lista.insiraNaPosicao("B", 2);
		lista.recebaVisitante(visitante);
		assertEquals("ABC", visitante.retorneDados());
	}

	@Test
	public void testeRemovaDaPosicao() {
		lista.insiraNoFim("A");
		lista.insiraNaPosicao("B", 2);
		lista.removaDaPosicao(2);
		lista.insiraNaPosicao("D", 2);
		lista.recebaVisitante(visitante);
		assertEquals("AD", visitante.retorneDados());
	}

	@Test
	public void testeRemovaDaPosicaoExcecao() {
		try {
			lista.removaDaPosicao(1);
			fail("Lista vazia");
		} catch (final ExcecaoEstruturaVazia e) {
		}
		lista.insiraNoFim("A");
		try {
			lista.removaDaPosicao(2);
			lista.removaDaPosicao(-1);
			fail("Posicao nao encontrada.");
		} catch (final ExcecaoPosicaoIlegal e) {
		}
	}

	@Test
	public void testeRemovaDoFim() {
		lista.insiraNoInicio("A");
		lista.insiraNoFim("C");
		lista.insiraNaPosicao("B", 2);
		lista.insiraNoFim("D");
		lista.removaDoFim();
		lista.recebaVisitante(visitante);
		assertEquals("ABC", visitante.retorneDados());
	}

	@Test
	public void testeRemovaDoFimExcecao() {
		try {
			lista.removaDoFim();
			lista.insiraNoFim("A");
			lista.removaDoFim();
			lista.removaDoFim();
			fail("Lista vazia");
		} catch (final ExcecaoEstruturaVazia e) {
		}
	}

	@Test
	public void testeRemovaDoInicio() {
		lista.insiraNoInicio("A");
		lista.insiraNoFim("B");
		lista.insiraNaPosicao("C", 3);
		lista.insiraNoFim("D");
		lista.removaDoInicio();
		lista.recebaVisitante(visitante);
		assertEquals("BCD", visitante.retorneDados());
	}

	@Test
	public void testeRemovaDoInicioExcecao() {
		try {
			lista.removaDoInicio();
			fail("Lista vazia");
		} catch (final ExcecaoEstruturaVazia e) {
		}
	}

	@Test
	public void testeRemovaIguais() {
		lista.insiraNoInicio("A");
		lista.insiraNoFim("B");
		lista.insiraNaPosicao("B", 2);
		lista.insiraNoFim("C");
		lista.removaIguais("B").recebaVisitante(visitante);
		assertEquals("BB", visitante.retorneDados());
		visitante.limpeDados();
		lista.recebaVisitante(visitante);
		assertEquals("AC", visitante.retorneDados());
	}

	@Test
	public void testeRemovaIguaisExcecao() {
		try {
			lista.removaIguais("A");
			fail("Lista vazia");
		} catch (final ExcecaoEstruturaVazia e) {
		}
	}

	@Test
	public void testeRemovaIgual() {
		lista.insiraNoInicio("A");
		lista.insiraNoFim("B");
		lista.insiraNaPosicao("B", 2);
		lista.insiraNoFim("C");
		lista.removaIguais("B").recebaVisitante(visitante);
		assertEquals("BB", visitante.retorneDados());
		visitante.limpeDados();
		lista.recebaVisitante(visitante);
		assertEquals("AC", visitante.retorneDados());
	}

	@Test
	public void testeRemovaIgualExcecao() {
		try {
			lista.removaIgual("A");
			fail("Lista vazia");
		} catch (final ExcecaoEstruturaVazia e) {
		}
	}

	@Test
	public void testeRetorneDaPosicao() {
		lista.insiraNoInicio("A");
		lista.insiraNoFim("B");
		lista.insiraNoFim("C");
		assertEquals("B", lista.retorneDaPosicao(2));
		lista.recebaVisitante(visitante);
		assertEquals("ABC", visitante.retorneDados());
	}

	@Test
	public void testeRetorneDaPosicaoExcecao() {
		try {
			lista.retorneDaPosicao(2);
			fail("Lista vazia");
		} catch (final ExcecaoEstruturaVazia e) {
		}

	}

	@Test
	public void testeRetorneDoFim() {
		try {
			lista.retorneDoFim();
			fail("Lista vazia");
		} catch (final ExcecaoEstruturaVazia e) {
		}
		lista.insiraNoInicio("A");
		lista.insiraNoFim("C");
		lista.insiraNaPosicao("B", 2);
		assertEquals("C", lista.retorneDoFim());
		lista.removaDoFim();
		assertEquals("B", lista.retorneDoFim());
	}

	@Test
	public void testeRetorneDoInicio() {
		lista.insiraNoInicio("A");
		lista.insiraNoFim("C");
		lista.insiraNaPosicao("B", 2);
		assertEquals("A", lista.retorneDoInicio());
	}

	@Test
	public void testeRetorneDoInicioExcecao() {
		try {
			lista.retorneDoInicio();
			fail("Lista vazia");
		} catch (final ExcecaoEstruturaVazia e) {
		}
	}

	@Test
	public void testeRetorneIterador() {
		Iterador<String> iterador = lista.retorneIterador();
		lista.insiraNoInicio("A");
		iterador = lista.retorneIterador();
		assertTrue(iterador.temMais());
		assertEquals("A", iterador.retorneProximo());
		lista.insiraNoFim("B");
		lista.insiraNoFim("C");
		lista.removaDoInicio();
		iterador = lista.retorneIterador();
		assertTrue(iterador.temMais());
		assertEquals("B", iterador.retorneProximo());
		assertTrue(iterador.temMais());
		assertEquals("C", iterador.retorneProximo());
		lista.removaDoFim();
		lista.removaDoFim();
		assertTrue(lista.estaVazia());
		iterador = lista.retorneIterador();
		assertFalse(iterador.temMais());
	}

	@Test
	public void testeRetorneIteradorExcecao() {
		final Iterador<String> iterador = lista.retorneIterador();
		assertFalse(iterador.temMais());
		try {
			iterador.retorneProximo();
			fail("Lista vazia");
		} catch (final ExcecaoOperacaoIlegal e) {
		}
	}

	@Test
	public void testeRetorneTamanho() {
		assertEquals(0, lista.retorneTamanho());
		lista.insiraNoInicio("A");
		assertEquals(1, lista.retorneTamanho());
		lista.insiraNoFim("A");
		assertEquals(2, lista.retorneTamanho());
		lista.removaDoInicio();
		assertEquals(1, lista.retorneTamanho());
	}

	@Test
	public void testeSubstituaDaPosicao() {
		lista.insiraNoFim("A");
		lista.insiraNaPosicao("B", 2);
		lista.substituaDaPosicao("C", 2);
		lista.recebaVisitante(visitante);
		assertEquals("AC", visitante.retorneDados());
	}

	@Test
	public void testeSubstituaDaPosicaoExcecao() {
		try {
			lista.substituaDaPosicao("B", 2);
			fail("Lista vazia");
		} catch (final ExcecaoEstruturaVazia e) {
		}
		lista.insiraNoFim("A");
		try {
			lista.substituaDaPosicao("B", 2);
			lista.substituaDaPosicao("B", -1);
			fail("Posicao Legal");
		} catch (final ExcecaoPosicaoIlegal e) {
		}
	}

	@Test
	public void testeSubstituaDoFim() {
		lista.insiraNoFim("A");
		lista.insiraNoFim("B");
		lista.substituaDoFim("C");
		lista.recebaVisitante(visitante);
		assertEquals("AC", visitante.retorneDados());
	}

	@Test
	public void testeSubstituaDoFimExcecao() {
		try {
			lista.substituaDoFim("B");
			fail("Lista vazia");
		} catch (final ExcecaoEstruturaVazia e) {
		}
	}

	@Test
	public void testeSubstituaDoInicio() {
		lista.insiraNoInicio("A");
		lista.insiraNoFim("C");
		lista.substituaDoInicio("B");
		lista.recebaVisitante(visitante);
		assertEquals("BC", visitante.retorneDados());
	}

	@Test
	public void testeSubstituaDoInicioExcecao() {
		try {
			lista.substituaDoInicio("A");
			fail("Lista vazia");
		} catch (final ExcecaoEstruturaVazia e) {
		}

	}

	@Test
	public void testeTemIguais() {
		lista.insiraNoInicio("A");
		lista.insiraNoFim("C");
		lista.insiraNaPosicao("B", 2);
		lista.insiraNaPosicao("B", 2);
		lista.temIguais("C").recebaVisitante(visitante);
		assertEquals("C", visitante.retorneDados());
		visitante.limpeDados();
		lista.temIguais("B").recebaVisitante(visitante);
		assertEquals("BB", visitante.retorneDados());
	}

	@Test
	public void testeTemIgual() {
		lista.insiraNoInicio("A");
		lista.insiraNoFim("C");
		lista.insiraNaPosicao("B", 2);
		assertEquals("B", lista.temIgual("B"));
		assertEquals(null, lista.temIgual("D"));
	}

}