package testes.pilhaBaseadaEmLER;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import ine5408.excecoes.ExcecaoEstruturaVazia;

import org.junit.Before;
import org.junit.Test;

import pilha.PilhaBaseadaEmLER;

public class TestesProprios {

	PilhaBaseadaEmLER<String> pilha;

	@Before
	public void prepararTestes() {
		pilha = new PilhaBaseadaEmLER<String>();
	}

	@Test
	public void testeDesempilhe() {
		pilha.empilhe("a");
		pilha.empilhe("b");
		pilha.empilhe("c");
		assertEquals("c", pilha.desempilhe());
		assertEquals("b", pilha.desempilhe());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeDesempilheDePilhaVaziaLancaExcecao() {
		pilha.desempilhe();
	}

	@Test
	public void testeEmpilhe() {
		pilha.empilhe("a");
		pilha.empilhe("b");
		assertEquals("b", pilha.retorneTopo());
		pilha.empilhe("c");
		assertEquals("c", pilha.retorneTopo());
	}

	@Test
	public void testeEstaCheia() {
		for (int i = 0; i < 1000; i++) {
			pilha.empilhe("a");
			pilha.empilhe("b");
			pilha.empilhe("c");
		}
		assertFalse(pilha.estaCheia());
	}

	@Test
	public void testeEstaVazia() {
		assertTrue(pilha.estaVazia());
		pilha.empilhe("a");
		assertFalse(pilha.estaVazia());
		pilha.desempilhe();
		assertTrue(pilha.estaVazia());
	}

	@Test
	public void testeEsvazie() {
		pilha.empilhe("a");
		pilha.empilhe("b");
		pilha.empilhe("c");
		assertFalse(pilha.estaVazia());
		pilha.esvazie();
		assertTrue(pilha.estaVazia());
	}

	@Test
	public void testeRetorneDoFim() {
		pilha.empilhe("a");
		pilha.empilhe("b");
		pilha.empilhe("c");
		assertEquals("a", pilha.retorneDoFim());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRetorneDoFimDePilhaVaziaLancaExcecao() {
		pilha.retorneDoFim();
	}

	@Test
	public void testeRetorneDoInicio() {
		pilha.empilhe("a");
		pilha.empilhe("b");
		assertEquals("b", pilha.retorneDoInicio());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRetorneDoInicioDePilhaVaziaLancaExcecao() {
		pilha.retorneDoInicio();
	}

	@Test
	public void testeRetorneTamanho() {
		pilha.empilhe("a");
		pilha.empilhe("b");
		pilha.empilhe("c");
		pilha.empilhe("d");
		assertEquals(4, pilha.retorneTamanho());
		pilha.desempilhe();
		assertEquals(3, pilha.retorneTamanho());
	}

	@Test
	public void testeRetorneTopo() {
		pilha.empilhe("a");
		assertEquals("a", pilha.retorneTopo());
		pilha.empilhe("b");
		assertEquals("b", pilha.retorneTopo());
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void testeRetorneTopoDePilhaVaziaLancaExcecao() {
		pilha.retorneTopo();
	}

}
