package testes.tabelaDeHash;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import ine5408.excecoes.ExcecaoEstruturaVazia;
import ine5408.excecoes.ExcecaoOperacaoIlegal;
import ine5408.lineares.Lista;
import ine5408.th.FuncaoG;
import ine5408.th.TabelaDeHash;

import org.junit.Before;
import org.junit.Test;

import tabelaDeHash.FuncaoGModulo;
import testes.VisitanteConcatenador;

public class TestesDeTabelaDeHash {

	private FuncaoG f = new FuncaoGModulo(10);
	private TabelaDeHash<Integer, String> tabela;

	@Test(expected = ExcecaoOperacaoIlegal.class)
	public void naoSePodeInserirDoisDadosComAMesmaChave() {
		tabela.insira(5, "boi");
		tabela.insira(5, "vaca");
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void naoSePodeRemoverElementoDeTabelaVazia() {
		tabela.remova(10);
	}

	@Test(expected = ExcecaoOperacaoIlegal.class)
	public void naoSePodeSubstituirElementoQueNaoEstaNaTabela() {
		tabela.insira(4, "vaca");
		tabela.substitua(5, "boi");
	}

	@Test(expected = ExcecaoEstruturaVazia.class)
	public void naoSePodeSubstituirSeTabelaEstaVazia() {
		tabela.substitua(5, "boi");
	}

	@Before
	public void prepararTestes() {
		tabela = new tabelaDeHash.TabelaDeHash<Integer, String>(f);
	}

	@Test
	public void removerElementoNaoExistenteRetornaNull() {
		tabela.insira(14, "quatorze");
		tabela.insira(15, "quinze");

		assertEquals(null, tabela.remova(13));
	}

	@Test
	public void testeEstaCheia() {
		assertFalse(tabela.estaCheia());

		for (int i = 0; i < 1000; i++)
			tabela.insira(i, "boi");

		assertFalse(tabela.estaCheia());
	}

	@Test
	public void testeEstaVazia() {
		assertTrue(tabela.estaVazia());
	}

	@Test
	public void testeEsvazie() {
		tabela.insira(10, "vaca");
		tabela.insira(11, "vaca");
		tabela.insira(0, "vaca");

		assertFalse(tabela.estaVazia());
		tabela.esvazie();

		assertTrue(tabela.estaVazia());

	}

	@Test
	public void testeG() {
		assertEquals(5, tabela.g(5));
		assertEquals(5, tabela.g(15));
		assertEquals(5, tabela.g(25));
		assertEquals(5, tabela.g(10102315));
	}

	@Test
	public void testeH() {
		assertEquals(tabela.g(tabela.f(5)), tabela.h(5));
		assertEquals(tabela.g(tabela.f(77)), tabela.h(77));
	}

	@Test
	public void testeInsira() {
		tabela.insira(5, "boi");
		assertEquals("boi", tabela.retorne(5));
	}

	@Test
	public void testeQueFEhHashCode() {
		final Integer n = new Integer("5");

		assertEquals(n.hashCode(), tabela.f(n));
	}

	public void testeQueRetorneElementoNaoExistenteRetornaNull() {
		tabela.insira(10, "dez");

		assertEquals(null, tabela.retorne(11));
	}

	@Test
	public void testeRecebaVisitante() {
		tabela.insira(1, "boi ");
		tabela.insira(2, "vaca ");
		tabela.insira(3, "touro ");

		VisitanteConcatenador v = new VisitanteConcatenador();
		tabela.recebaVisitante(v);

		assertEquals("boi vaca touro ", v.retorneDados());
	}

	@Test
	public void testeRemova() {
		tabela.insira(5, "boi");
		assertFalse(tabela.estaVazia());
		tabela.remova(5);
		assertTrue(tabela.estaVazia());
	}

	@Test
	public void testeRetorne() {
		tabela.insira(10, "vaca");
		assertEquals("vaca", tabela.retorne(10));
	}

	@Test
	public void testeRetorneChaves() {
		tabela.insira(1, "vaca");
		tabela.insira(3, "vaca");
		tabela.insira(7, "vaca");
		tabela.insira(8, "vaca");

		Lista<Integer> lista = tabela.retorneChaves();

		assertEquals(4, lista.retorneTamanho());
		assertTrue(lista.pertence(1));
		assertTrue(lista.pertence(7));
		assertTrue(lista.pertence(8));
		assertTrue(lista.pertence(3));
	}

	@Test
	public void testeRetorneDados() {
		tabela.insira(1, "vaca");
		tabela.insira(3, "boi");
		tabela.insira(7, "touro");
		tabela.insira(8, "egua");

		Lista<String> lista = tabela.retorneDados();

		assertEquals(4, lista.retorneTamanho());
		assertTrue(lista.pertence("vaca"));
		assertTrue(lista.pertence("touro"));
		assertTrue(lista.pertence("boi"));
		assertTrue(lista.pertence("egua"));
	}

	@Test
	public void testeRetorneM() {
		assertEquals(10, tabela.retorneM());
	}

	@Test
	public void testeRetorneTamanho() {
		assertEquals(0, tabela.retorneTamanho());
		tabela.insira(5, "a");
		assertEquals(1, tabela.retorneTamanho());
		tabela.insira(6, "a");
		tabela.insira(7, "a");
		tabela.insira(0, "a");
		assertEquals(4, tabela.retorneTamanho());

		tabela.esvazie();
		assertEquals(0, tabela.retorneTamanho());
	}

	@Test
	public void testeRetorneTamanhoColisoesPorEntrada() {
		tabela.insira(6, "a");
		tabela.insira(16, "a");
		tabela.insira(26, "a");
		tabela.insira(7, "a");
		tabela.insira(157, "a");
		tabela.insira(9, "a");

		int m = f.getM();
		int[] certo = new int[m];
		certo[6] = 3;
		certo[7] = 2;
		certo[9] = 1;
		certo[1] = 0;
		certo[0] = certo[2] = certo[3] = certo[4] = certo[5] = certo[8] = 0;
		for (int i = 10; i < m; i++)
			certo[i] = 0;

		int[] colisoes = tabela.retorneTamanhoColisoesPorEntrada();
		assertArrayEquals(certo, colisoes);
	}

	@Test
	public void testeSubstitua() {
		tabela.insira(5, "vaca");
		assertEquals("vaca", tabela.retorne(5));

		tabela.substitua(5, "boi");
		assertEquals("boi", tabela.retorne(5));
	}

}
